#include <windows.h>
#include <windowsx.h>
#include<fstream>
#include <iostream>

#define _USE_MATH_DEFINES
#include <math.h>


#define WIN32_LEAN_AND_MEAN
#include "resource1.h"
#include "util.h"
#include "matrix.h"



using namespace std;

HINSTANCE hInst;
HWND hWnd;
HWND hwndButtonClasseButton;
HDC hdc;

Matrix M;


int largeur ;
int hauteur ;
int nbPixel ;

unsigned char **origR;
unsigned char **origV;
unsigned char **origB;

unsigned char **resulR;
unsigned char **resulV;
unsigned char **resulB;

ifstream *pnm_orig;
ofstream *pnm_final;

bool affich_dr = false;

bool interpolBilineaire;




void affiche_droite(bool ok){
	if(ok){
		for(int i=0; i<hauteur; i++){
 			for(int j=0; j<largeur; j++){
				SetPixel(hdc, j+largeur, i, RGB(resulR[i][j], resulV[i][j], resulB[i][j]));
			}
		}
	}
}

unsigned char interpolPPV( float x, float y, unsigned char **tab )
{
	int intx = (int)x , inty = (int)y;

    if(( x - intx ) >= 0.5f ) intx++;
	if(( y - inty ) >= 0.5f ) inty++;

	if(intx>=0 && intx<largeur && inty>=0 && inty<hauteur)
		return tab[inty][intx];
	else
		return 1; 
}

unsigned char interpolBili( float x, float y, unsigned char **tab){

	int intx = (int)x , inty = (int)y;
	int intxp = intx+1 , intyp = inty+1;
	float dx = x-intx, dy = y-inty;

	float Iinter1, Iinter2, Iresul;

	if(intx>=0 && intxp<largeur && inty>=0 && intyp<hauteur){
		
		Iinter1 = (1-dy)*tab[inty][intx] + dy*tab[intyp][intx];
		Iinter2 = (1-dy)*tab[inty][intxp] + dy*tab[intyp][intxp];

		Iresul = (1-dx)*Iinter1 + dx*Iinter2;


		return (unsigned char)Iresul;

	}else
		return 0;
}


void transformation(void){

	unsigned char **interR;
	unsigned char **interV;
	unsigned char **interB;
	
	interR = new unsigned char*[hauteur];
	interV = new unsigned char*[hauteur];
	interB = new unsigned char*[hauteur];
	for(int i=0; i<hauteur; i++){
		interR[i] = new unsigned char[largeur];
		interV[i] = new unsigned char[largeur];
		interB[i] = new unsigned char[largeur];
	}
				  
	for(int i=0; i<hauteur; i++){
 		for(int j=0; j<largeur; j++){
 			interR[i][j] = resulR[i][j];
			interV[i][j] = resulV[i][j];
			interB[i][j] = resulB[i][j];
 		}
	}			
	
	for(int i=0; i<hauteur; i++){
 			for(int j=0; j<largeur; j++){

				Vector V((float)j, (float)i, 1.f);
				V = M * V;

				if(!interpolBilineaire){
					resulR[i][j] = interpolPPV(V.getVal(0), V.getVal(1), interR);
					resulV[i][j] = interpolPPV(V.getVal(0), V.getVal(1), interV);
					resulB[i][j] = interpolPPV(V.getVal(0), V.getVal(1), interB);
				}else{
					resulR[i][j] = interpolBili(V.getVal(0), V.getVal(1), interR);
					resulV[i][j] = interpolBili(V.getVal(0), V.getVal(1), interV);
					resulB[i][j] = interpolBili(V.getVal(0), V.getVal(1), interB);
				}

			}
	}
}






BOOL Explorateur( LPSTR& nFile , BOOL save )
{
	
	nFile=new CHAR[256];
	nFile[0]='\0';

	OPENFILENAME file;

	
	file.lStructSize=sizeof(OPENFILENAME);
	file.hwndOwner=hWnd;   
	file.hInstance=hInst;	
	file.lpstrFilter="Image PNM\0*.ppm\0";
	file.lpstrCustomFilter=NULL;
	file.nFilterIndex=0;
	file.lpstrFile=nFile;  
	file.nMaxFile=256;
	file.lpstrFileTitle=NULL;
	file.lpstrInitialDir=NULL;
	file.lpfnHook=NULL;
	file.lpstrDefExt=NULL;
	file.lpTemplateName=NULL;
	file.lCustData=(LPARAM)0;

	if( ! save )
	{	
		file.lpstrTitle="Ouverture du fichier ...";
		file.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST; 
		return GetOpenFileName( &file );
	}
	else
	{
		file.lpstrTitle = "Enregistrement du fichier ...";
		file.Flags = OFN_EXPLORER | OFN_OVERWRITEPROMPT; 
		return GetSaveFileName( &file );
	}

}

BOOL CALLBACK MsgDialogProc( HWND hwnd , UINT msg , WPARAM wParam , LPARAM lParam )
{

	switch( msg )
	{
		case WM_INITDIALOG:
			CheckRadioButton( hwnd , IDC_PPV , IDC_BILI , IDC_PPV );
            return TRUE;

		case WM_COMMAND :
          switch (LOWORD (wParam))
          {
          case IDOK : 
			  float angle,tx,ty;
			
			  char str[10];
			 
			  GetDlgItemText( hwnd , IDC_ANGLE , str , 10 );
			  angle = (float)atof( str );
			  GetDlgItemText( hwnd , IDC_TRANSX , str , 10 );
			  tx = (float)atof( str );
			  GetDlgItemText( hwnd , IDC_TRANSY , str , 10 );
			  ty = (float)atof( str );


			  affich_dr = true;
			  
			  if( IsDlgButtonChecked( hwnd , IDC_PPV ))		interpolBilineaire = false;
			  if( IsDlgButtonChecked( hwnd , IDC_BILI ))	interpolBilineaire = true;

			  M.identity();
			  M.rotation( largeur/2.f , hauteur/2.f , (float)((angle*M_PI)/ 180.f) );
			  M.translation( tx , ty );

			  transformation();

			 
			

          case IDCANCEL :
               EndDialog (hwnd, 0) ;
               return TRUE ;
          }
          break ;
	}
	return FALSE;
}




LRESULT CALLBACK MsgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	
	LPSTR fichier = NULL;
	PAINTSTRUCT ps;
	ZeroMemory( &ps , sizeof(ps) );
	
	ps.fErase = true;

	

	switch (message) 
	{
	case WM_COMMAND:
		

		switch(wParam){
			case ID_QUIT :
				
				PostQuitMessage( 0 );
				break;
			case ID_OUVRIR :
				if(Explorateur(fichier, false)){
				
					pnm_orig = new ifstream((char *)fichier,std::ios::binary);
					char ch;

					for(int i=0; i<2; i++){
						pnm_orig->get(ch);
						
						while(ch != '\n'){
						pnm_orig->get(ch);
						  
						}
					}
					  
					char buf[3];
					char buf2[3];
					pnm_orig->read(buf, 3);
					pnm_orig->get();
					pnm_orig->read(buf2, 3);
					  
					largeur = atoi(buf);
					hauteur = atoi(buf2);
		
					nbPixel = largeur * hauteur;
					  
					  
					pnm_orig->get(ch);
					pnm_orig->get(ch);
					while(ch != '\n'){
 						pnm_orig->get(ch);
					}
					  
					  
					origR = new unsigned char*[hauteur];
					origV = new unsigned char*[hauteur];
					origB = new unsigned char*[hauteur];
					for(int i=0; i<hauteur; i++){
						origR[i] = new unsigned char[largeur];
						origV[i] = new unsigned char[largeur];
						origB[i] = new unsigned char[largeur];
						
						
					}
					  
					for(int i=0; i<hauteur; i++){
 						for(int j=0; j<largeur; j++){
 							pnm_orig->read((char *)&origR[i][j],1);
 							pnm_orig->read((char *)&origV[i][j],1);
							pnm_orig->read((char *)&origB[i][j],1);
						}
					}	


					resulR = new unsigned char*[hauteur];
					resulV = new unsigned char*[hauteur];
					resulB = new unsigned char*[hauteur];
					for(int i=0; i<hauteur; i++){
						resulR[i] = new unsigned char[largeur];
						resulV[i] = new unsigned char[largeur];
						resulB[i] = new unsigned char[largeur];
						
						
					}
					  
					for(int i=0; i<hauteur; i++){
 						for(int j=0; j<largeur; j++){
 							resulR[i][j] = origR[i][j];
							resulV[i][j] = origV[i][j];
							resulB[i][j] = origB[i][j];
	 						
						}
					}				
					delete [] fichier;
					InvalidateRect( hWnd , NULL , true );

				}

				break;
			case ID_SAVE :
				if(Explorateur(fichier, true)){
					pnm_final = new ofstream(fichier , ios::out | ios::binary);

					

					*pnm_final<< "P6" << endl;
					*pnm_final<< "# " << fichier << " image transforme" <<endl;
					*pnm_final<< largeur << " " << hauteur <<endl;
					*pnm_final<< "255" <<endl;

					for( int j=0 ; j<hauteur ; j++ ){
						for( int i =0 ; i<largeur ; i++ ){
							*pnm_final << resulR[j][i] << resulV[j][i] << resulB[j][i];
						}
					}
					delete [] fichier;
					pnm_final->close();
				}
				break;
			case ID_TRANSFO :
				DialogBox(hInst, MAKEINTRESOURCE(IDD_DIALOG1), hWnd , MsgDialogProc );
				InvalidateRect( hWnd , NULL , true );
				break;


			case ID_ABOUT :
				MessageBox(hWnd, "Transformation d'une image par rotation et translation\n \t by AkouFen SoftW@re", "About",
							MB_ICONINFORMATION | MB_OK);
				break;
		
		}
		break;

	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		
		for(int i=0; i<hauteur; i++){
 			for(int j=0; j<largeur; j++){
				SetPixel(hdc, j, i, RGB(origR[i][j], origV[i][j], origB[i][j]));
			}
		}
		affiche_droite(affich_dr);
	
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	case WM_QUIT:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}




int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hpInst, LPSTR CmdLine, int CmdShow){
	hWnd = NULL;
	
	
	WNDCLASSEX wc;
	wc.lpszClassName = "MyClass";
    wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = MsgProc;
    wc.hInstance     = hInst;
    wc.hIcon         = NULL;
    wc.hIconSm       = NULL;
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)COLOR_BACKGROUND;
    wc.lpszMenuName  = (LPCSTR)IDR_MENU1;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;

	if(!RegisterClassEx(&wc)) return 1;

	hWnd = CreateWindow("MyClass", "Transfo Image", WS_OVERLAPPEDWINDOW|WS_VISIBLE, 100, 100, 1000, 500, NULL, NULL, hInst, NULL);
	if(hWnd==NULL) return 1;


	ShowWindow( hWnd, SW_SHOWDEFAULT );
    UpdateWindow( hWnd );



	MSG msg;
	ZeroMemory( &msg, sizeof(msg) );
	

	while( msg.message!=WM_QUIT ) {
		if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) ) {
			TranslateMessage( &msg );
            DispatchMessage( &msg );
		}
	}

	UnregisterClass( "MyClass", hInst);
	return (int) msg.wParam;

}