	
#include "matrix.h"

Matrix::Matrix(){
	for( int i=0 ; i<3 ; i++){
		for( int j=0 ; j<3 ; j++){
			mat[i][j] = 0.f;
		}
	}
}

Matrix::Matrix( float m11 , float m12 , float m13 ,
				float m21 , float m22 , float m23 ,
				float m31 , float m32 , float m33 ){
	
	mat[0][0] = m11; mat[0][1] = m12; mat[0][2] = m13;
	mat[1][0] = m21; mat[1][1] = m22; mat[1][2] = m23;
	mat[2][0] = m31; mat[2][1] = m32; mat[2][2] = m33;
}

Matrix::Matrix( const Matrix& M )
{
	for( int i=0 ; i<3 ; i++){
		for( int j=0 ; j<3 ; j++){
			mat[i][j] = M.mat[i][j];
		}
	}
}

Matrix::~Matrix() {}

void Matrix::init( float val )
{
	for( int i=0 ; i<3 ; i++){
		for( int j=0 ; j<3 ; j++){
			mat[i][j] = val;
		}
	}
}

void Matrix::identity()
{
	for( int i=0 ; i<3 ; i++){
		for( int j=0 ; j<3 ; j++){
			if( i == j ) mat[i][j] = 1.f;
			else mat[i][j] = 0.f;
		}
	}
}

void Matrix::translation( float x , float y )
{
	Matrix M(	1.f , 0.f , -x ,
				0.f , 1.f , y ,
				0.f , 0.f , 1.f );

	//M = !M;

	(*this) = M * (*this);
}

void Matrix::zoom( float x , float y )
{
	Matrix M(	x , 0.f , 0.f ,
				0.f , y , 0.f ,
				0.f , 0.f , 1.f );

	(*this) = M * (*this);
}

void Matrix::rotation( float cX , float cY , float angle )
{
	translation( cX , -cY );

	Matrix M(	cos(-angle) , -sin(-angle) , 0.f ,
				sin(-angle) , cos(-angle) , 0.f ,
				0.f , 0.f , 1.f );

	M = !M;

	(*this) = M * (*this);

	translation( -cX , cY );
}

void Matrix::setVal( int l , int c , float val )
{
	if( l>=0 && l<3 && c>=0 && c<3 )
		mat[l][c] = val;
}

const float Matrix::getVal( int l , int c ){
	if( l>=0 && l<3 && c>=0 && c<3 )
		return mat[l][c];
	else
		return -1.f;
}


void Matrix::affiche(){
	for( int i=0 ; i<3 ; i++){
		for( int j=0 ; j<3 ; j++){
			cout<<mat[i][j]<<'\t';
		}
		cout<<endl;
	}
}
	


Matrix& Matrix::operator= ( const Matrix& M )
{
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			mat[i][j] = M.mat[i][j];

	return (*this);
}




Matrix operator+ ( const Matrix& M1 , const Matrix& M2 )
{
	Matrix M;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			M.mat[i][j] = M1.mat[i][j] + M2.mat[i][j];

	return M;
}

Matrix operator- ( const Matrix& M1 , const Matrix& M2 )
{
	Matrix M;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			M.mat[i][j] = M1.mat[i][j] - M2.mat[i][j];

	return M;
}


Matrix operator* ( const Matrix& M1 , const Matrix& M2 )
{
	Matrix M;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			for( int k=0; k<3 ; k++)
				M.mat[i][j] += M1.mat[i][k] * M2.mat[k][j];

	return M;
}


Matrix operator* ( const Matrix& M , float val )
{
	Matrix Mat;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			Mat.mat[i][j] = M.mat[i][j] * val;

	return Mat;
}

Matrix operator* ( float val , const Matrix& M )
{
	Matrix Mat;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			Mat.mat[i][j] = M.mat[i][j] * val;

	return Mat;
}

Matrix operator/ ( const Matrix& M , float val )
{
	Matrix Mat;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			Mat.mat[i][j] = M.mat[i][j] / val;

	return Mat;
}

Matrix operator! (const Matrix& M )
{
	Matrix Mat;
	for( int i=0 ; i<3 ; i++)
		for( int j=0 ; j<3 ; j++)
			Mat.mat[i][j] = M.mat[j][i];

	float det = (M.mat[0][0]*M.mat[1][1]*M.mat[2][2]) + 
				(M.mat[1][0]*M.mat[2][1]*M.mat[0][2]) +
				(M.mat[2][0]*M.mat[0][1]*M.mat[1][2]) -
				(M.mat[0][2]*M.mat[1][1]*M.mat[2][0]) -
				(M.mat[1][2]*M.mat[2][1]*M.mat[0][0]) -
				(M.mat[2][2]*M.mat[0][1]*M.mat[1][0]);

	if( det != 0 ) Mat = (float)(1.f/det) * Mat;

	return Mat;
}

Vector operator* ( const Matrix& M , const Vector& V )
{
	Vector result;

	for( int i=0 ; i<3 ;i++ )
		for( int j=0 ; j<3 ; j++ )
			result.vect[i] += M.mat[i][j] * V.vect[j];

	return result;
}

Vector::Vector()
{
	for( int i=0 ; i<3 ; i++ )
		vect[i] = 0.f;
}

Vector::Vector( float v1 , float v2 , float v3 )
{
	vect[0] = v1;
	vect[1] = v2;
	vect[2] = v3;
}
	
Vector::Vector( const Vector& V )
{
	for( int i=0 ; i<3 ; i++ )
		vect[i] = V.vect[i];
}

Vector::~Vector() {}



void Vector::init( float v1 , float v2 , float v3 )
{
	vect[0] = v1;
	vect[1] = v2;
	vect[2] = v3;
}

void Vector::setVal( int pos , float val )
{
	if( pos>=0 && pos<3 )
		vect[pos] = val; 
}

const float Vector::getVal( int pos )
{
	if( pos>=0 && pos<3 )
		return vect[pos];
	else
		return -1.f;
}

void Vector::affiche(){
	for( int i=0 ; i<3 ; i++ )
		cout<<vect[i]<<" "; 

	cout<<endl;
}
